@@ -189,6 +189,13 @@ def format_line_mask_option(entry_value, wave_array):
     return formatted_value
 
 
+def mult_err_propagation(nominal_array, err_array, result):
+
+    err_result = result * np.sqrt(np.sum(np.power(err_array/nominal_array, 2)))
+
+    return err_result
+
+
 class EmissionFitting:
 
     """Class to measure emission line fluxes and fit them as gaussian curves"""
@@ -225,7 +232,7 @@ class EmissionFitting:
         self.m_cont, self.n_cont = None, None
         self.amp, self.center, self.sigma = None, None, None
         self.amp_err, self.center_err, self.sigma_err = None, None, None
-        self.z_line = 0
+        self.z_line = None
         self.v_r, self.v_r_err = None, None
         self.pixel_vel = None
         self.sigma_vel, self.sigma_vel_err = None, None
@@ -362,9 +369,6 @@ class EmissionFitting:
         n_comps = mixtureComponents.size
         ion_arr, theoWave_arr, latexLabel_arr = label_decomposition(mixtureComponents, comp_dict=user_conf)
 
-        # Line redshift
-        self.z_line = self.peak_wave / theoWave_arr - 1
-
         # Pixel velocity
         self.pixel_vel = c_KMpS * self.pixelWidth/self.peak_wave
 
@@ -374,8 +378,14 @@ class EmissionFitting:
         else:
             ref_wave = np.array([self.peak_wave], ndmin=1)
 
+        # Line redshift
+        # TODO calculate z_line using the label reference
+        self.z_line = self.peak_wave/ref_wave[0] - 1
+
         # Kinematics corrections
-        self.sigma_instr = k_FWHM/self.inst_FWHM if not np.isnan(self.inst_FWHM) else np.nan
+        self.sigma_instr = k_FWHM/self.inst_FWHM if not np.isnan(self.inst_FWHM) else None
+
+        # TODO create the parameters in advance in order to remove the components order requirement
 
         # Define fitting params for each component
         fit_model = Model(linear_model)
@@ -397,8 +407,7 @@ class EmissionFitting:
             # Amplitude default configuration changes according and emission or absorption feature
             AMP_PAR = self._AMP_PAR if self._emission_check else self._AMP_ABS_PAR
 
-            # Define the curve parameters
-            #TODO include the normalization here
+            # Define the curve parameters # TODO include the normalization here
             self.define_param(fit_model, comp, 'amp', self.peak_flux - self.cont, AMP_PAR, user_conf)
             self.define_param(fit_model, comp, 'center', ref_wave[idx_comp], self._CENTER_PAR, user_conf, z_cor=(1+z_obj))
             self.define_param(fit_model, comp, 'sigma', 1.0, self._SIG_PAR, user_conf)
@@ -473,6 +482,9 @@ class EmissionFitting:
             self.FWHM_g[i] = k_FWHM * self.sigma_vel[i]
             self.sigma_thermal[i] = np.sqrt(k_Boltzmann * self.temp_line / self._atomic_mass_dict[ion_arr[i][:-1]]) / 1000
 
+            # Check parameters error progragation from the lmfit parameter
+            self.error_propagation_check(i, line)
+
         if self.blended_check:
             self.eqw, self.eqw_err = eqw_g, eqwErr_g
         else:
@@ -550,6 +562,18 @@ class EmissionFitting:
 
         return
 
+    def error_propagation_check(self, idx_line, line_label):
+
+        # Check gaussian flux error
+        if (self.gauss_err[idx_line] == 0.0) and (self.amp_err[idx_line] != 0.0) and (self.sigma_err[idx_line] != 0.0):
+            self.gauss_err[idx_line] = mult_err_propagation(np.array([self.amp[idx_line], self.sigma[idx_line]]),
+                                                            np.array([self.amp_err[idx_line], self.sigma_err[idx_line]]),
+                                                            self.gauss_flux[idx_line])
+
+        # Check equivalent width error
+
+        return
+
     def velocity_percentiles_calculations(self, vel_array, line_flux, cont_flux):
 
         if vel_array.size > 15:
